/**
 * Global state for Telemetry instance (survives hot reloads)
 */
import type { TelemetryResponse } from './telemetry';
export declare const telemetry: {
    get: () => TelemetryResponse | undefined;
    set: (v: TelemetryResponse) => void;
};
//# sourceMappingURL=globals.d.ts.map